// Import D3 library and TopoJSON features
import * as d3 from 'd3';
import { feature } from 'topojson';

// Define different map projections
var projections = [
  { name: 'mercator', value: d3.geoMercator() }, // Mercator projection
  { name: 'orthographic', value: d3.geoOrthographic() }, // Orthographic projection
  { name: 'Us', value: d3.geoAlbersUsa() }, // Albers USA projection
];

// Set initial projection to orthographic
let projection = projections[1].value;

// Select SVG element
const svg = d3.select('svg');

// Create graticule lines
const graticule = d3.geoGraticule();

// Create path generator using the current projection
let pathGenerator = d3.geoPath(projection);

// Variable to store the currently selected country
let country;

// Append a group element for map elements
const g = svg.append('g');

// Append a sphere to represent the globe
g.append('path')
  .attr('class', 'sphere')
  .attr('d', pathGenerator({ type: 'Sphere' }));

// Append graticule lines to the map
g.append('path')
  .datum(graticule)
  .attr('class', 'graticule')
  .attr('d', pathGenerator);

// Enable dragging to rotate the globe
g.call(
  d3.drag().on('drag', (event, d) => {
    const rotate = projection.rotate();
    const k = 75 / projection.scale();
    projection.rotate([
      rotate[0] + event.dx * k,
      rotate[1] - event.dy * k,
    ]);
    pathGenerator = d3.geoPath().projection(projection);
    svg.selectAll('.graticule').attr('d', pathGenerator);
    svg.selectAll('.country').attr('d', pathGenerator);
  }),
);

// Function to update the map when projection changes
function update() {
  pathGenerator = d3.geoPath().projection(projection);
  svg.selectAll('path').attr('d', pathGenerator);
  svg.selectAll('path').transition().duration(750);
  svg
    .selectAll('.sphere')
    .attr('d', pathGenerator({ type: 'Sphere' }));
}

// Load data asynchronously
Promise.all([
  d3.tsv(
    'https://unpkg.com/world-atlas@1.1.4/world/110m.tsv',
  ),
  d3.json(
    'https://gist.githubusercontent.com/NandithaSura/d8a6cd0aea16ec595373a5e7d6470326/raw/4539378b648964e983390193c00152fa986be8a4/globe.json',
  ),
]).then(([tsvData, topoJSONdata]) => {
  // Create a mapping of country codes to country names
  const countryName = tsvData.reduce((accumulator, d) => {
    accumulator[d.iso_n3] = d.name;
    return accumulator;
  });

  // Extract features from the TopoJSON data
  const countries = feature(
    topoJSONdata,
    topoJSONdata.objects.countries,
  );

  // Append paths for each country
  g.selectAll('.country')
    .data(countries.features)
    .enter()
    .append('path')
    .attr('class', 'country')
    .attr('d', pathGenerator)
    .on('click', function (event, d) {
      country = countryName[d.id];
      if (
        country === 'United States' &&
        projection !== projections[0].value
      ) {
        projection = projections[0].value;
        update();
      } else {
        projection = projections[1].value;
        update();
      }
    });

  // Add labels to the countries
  g.selectAll('.country-label')
    .data(countries.features)
    .enter()
    .append('text')
    .attr('class', 'country-label')
    .attr('transform', function (d) {
      const centroid = pathGenerator.centroid(d);
      return `translate(${centroid[0]},${centroid[1]})`;
    })
    .attr('text-anchor', 'middle')
    .attr('dy', '.35em')
    .style('fill', 'black')
    .style('font-size', '3.5px')
    .style('font-family', 'Arial, sans-serif')
    .text((d) => countryName[d.id]);
});

// Enable zooming and panning
g.call(
  d3.zoom().on('zoom', (event) => {
    g.attr('transform', event.transform);
  }),
);
